# -*- coding: utf-8 -*-
# Project : flacon
# File : tracklist.py
# (c) Alexander Sokolov asokol@mail.ru
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.


from preferences import *
from flaconglobal import *

from PyQt4 import QtGui, QtCore
from PyQt4.QtCore import *
from PyQt4.QtGui import *
from errors import *

class AudioFileInfo(QObject):
   #****************************************************************
   #
   #****************************************************************
    def __init__(self, fileName):
        QObject.__init__(self)
        self._items = {}

        FileError.checkFile(fileName)

        shntool = qApp.preferences.value("Programs/shntool").toString()
        proc = tryPopen([unicode(shntool), 'info', unicode(fileName)], stdout=subprocess.PIPE, stderr=subprocess.STDOUT, close_fds=True)
        res = proc.wait()
        if res != 0:
            raise FileErrorBadAudio(fileName)

        for line in proc.stdout.readlines():
            name = QString(line).section(':', 0, 0).toUpper().trimmed()
            value =QString(line).section(':', 1).trimmed()
            self._items[name]=value;


   #****************************************************************
   #
   #****************************************************************
    def item(self, name):
        try:
            return QString(self._items[QString(name).toUpper()])
        except:
            return ""


   #****************************************************************
   #
   #****************************************************************
    def format(self):
        s = self.item('Handled by')

        if s == 'ape format module':  return AudioFormat.APE
        if s == 'flac format module': return AudioFormat.FLAC
        if s == 'wav format module':  return AudioFormat.WAV
        if s == 'wv format module':   return AudioFormat.WV
        if s == 'tta format module':  return AudioFormat.WV


class AudioFile(QObject):
   #****************************************************************
   #
   #****************************************************************
    def __init__(self, disc, fileName, info = None):
        QObject.__init__(self, disc)

        self._disc = disc
        fi = QFileInfo(fileName)

        self._fileName = fi.canonicalFilePath()

        if info:
            self._info = info
        else:
            self._info = AudioFileInfo(fileName)

        self._format = self._info.format()


   #****************************************************************
   #
   #****************************************************************
    def fileName(self):
        return self._fileName


   #****************************************************************
   #
   #****************************************************************
    def haveEmbededCue(self):
        return False


   #****************************************************************
   #
   #****************************************************************
    def format(self):
        return self._format


   #****************************************************************
   #
   #****************************************************************
    def cdQuality(self):
        return self._info.item("CD quality") == 'yes'



class ApeFile(AudioFile):
   #****************************************************************
   #
   #****************************************************************
    def __init__(self, disc, fileName, info = None):
        AudioFile.__init__(self, disc, fileName, info)



class FlacFile(AudioFile):
   #****************************************************************
   #
   #****************************************************************
    def __init__(self, disc, fileName, info = None):
        AudioFile.__init__(self, disc, fileName, info)



class WvFile(AudioFile):
   #****************************************************************
   #
   #****************************************************************
    def __init__(self, disc, fileName, info = None):
        AudioFile.__init__(self, disc, fileName)



class WavFile(AudioFile):
   #****************************************************************
   #
   #****************************************************************
    def __init__(self, disc, fileName, info = None):
        AudioFile.__init__(self, disc, fileName)


class TtaFile(AudioFile):
   #****************************************************************
   #
   #****************************************************************
    def __init__(self, disc, fileName, info = None):
        AudioFile.__init__(self, disc, fileName, info)


#****************************************************************
#
#****************************************************************
def getAudioFile(fileName, disc):
    audioInfo = AudioFileInfo(fileName)
    format = audioInfo.format()

#    if not audioFile.isFile():
#        return None

#    if not self._audioFile.isReadable():
#        return None

    if format == AudioFormat.APE:  return ApeFile(disc, fileName, audioInfo)
    if format == AudioFormat.FLAC: return FlacFile(disc, fileName, audioInfo)
    if format == AudioFormat.WV:   return WvFile(disc, fileName, audioInfo)
    if format == AudioFormat.WAV:  return WavFile(disc, fileName, audioInfo)

    return None
