# -*- coding: utf-8 -*-
# Project : flacon
# File : disc.py
# (c) Alexander Sokolov asokol@mail.ru
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.


#import string
import re

from preferences import *
from flaconglobal import *
from audiofiles import *

from PyQt4 import QtGui, QtCore
from PyQt4.QtCore import *
from errors import *



class Track():
   #****************************************************************
   #
   #****************************************************************
    def __init__(self, disc):
        self._disc = disc

        # Tags storrage structure:
        # +------+--------------------------------+
        # | role | +---------+------------------+ |
        # |      | | tagName | (value, encoded) | |
        # |      | | tagName | (value, encoded) | |
        # |      | |        ...                 | |
        # |      | +---------+------------------+ |
        # +------+--------------------------------+
        # | role | +---------+------------------+ |
        # |      | | tagName | (value, encoded) | |
        # |      | | tagName | (value, encoded) | |
        # |      | |        ...                 | |
        # |      | +---------+------------------+ |
        # +------+--------------------------------+
        #   ...
        #
        self._tags = {}
        self._trackNum = 0

        self._progress = -1
        self._status  = Statuses.NotRunning


   #****************************************************************
   #
   #****************************************************************
    def index(self):
        return self._disc._tracks.index(self)


   #****************************************************************
   #
   #****************************************************************
    def decode(self, value):
        if not isinstance(value, QString):
            return value

        codec = QTextCodec.codecForName(self._disc.codePage())
        if (codec):
            return codec.toUnicode(value)
        else:
            return value


   #****************************************************************
   #
   #****************************************************************
    def _tag(self, tagName, role=None):
        if not role:
            role = self._disc.currentRole()

        try:
            (value, encoded) = self._tags[role][tagName]
            if encoded:
                return value
            else:
                return self.decode(value)

        except KeyError, e:
            return QString()


   #****************************************************************
   #
   #****************************************************************
    def _setTag(self, tagName, value, encoded=True, role=None):
        if isinstance(value, QString):
            value = QString(value)

        if isinstance(value, QVariant):
            value = QVariant(value)

        if not role:
            role = self._disc.currentRole()

        if not role in self._tags:
            self._tags[role] = {}

        self._tags[role][tagName] = (value, encoded)

        if role == self._disc.currentRole():
            self._disc.trackChanged(self)


   #****************************************************************
   #
   #****************************************************************
    def artist(self):
        return self._tag('PERFORMER')


   #****************************************************************
   #
   #****************************************************************
    def setArtist(self, value, encoded=True, role=None):
        self._setTag('PERFORMER', value, encoded, role)


   #****************************************************************
   #
   #****************************************************************
    def album(self):
        return self._tag('ALBUM')


   #****************************************************************
   #
   #****************************************************************
    def setAlbum(self, value, encoded=True, role=None):
        self._setTag('ALBUM', value, encoded, role)



   #****************************************************************
   #
   #****************************************************************
    def title(self):
        return self._tag('TITLE')


   #****************************************************************
   #
   #****************************************************************
    def setTitle(self, value, encoded=True, role=None):
        self._setTag('TITLE', value, encoded, role)


   #****************************************************************
   #
   #****************************************************************
    def year(self):
        return self._tag('YEAR')


   #****************************************************************
   #
   #****************************************************************
    def setYear(self, value, encoded=True, role=None):
        try:
            self._setTag('YEAR', int(value), True, role)

        except ValueError, e:
            r = re.compile(r'(\d{1,4})').search(value)
            if (r):
                self._setTag('YEAR', int(r.groups()[0]), True, role)
            else:
                self._setTag('YEAR', 0, True, role)


   #****************************************************************
   #
   #****************************************************************
    def genre(self):
        return self._tag('GENRE')


   #****************************************************************
   #
   #****************************************************************
    def setGenre(self, value, encoded=True, role=None):
        self._setTag('GENRE', value, encoded, role)


   #****************************************************************
   #
   #****************************************************************
    def discId(self):
        return self._tag('DISCID')


   #****************************************************************
   #
   #****************************************************************
    def setDiscId(self, value, encoded=True, role=None):
        self._setTag('DISCID', value, encoded, role)


   #****************************************************************
   #
   #****************************************************************
    def trackNum(self):
        return int(self._trackNum)+ self._disc.startTrackNum() - 1


   #****************************************************************
   #
   #****************************************************************
    def setTrackNum(self, value):
        self._trackNum = value


   #****************************************************************
   # %N  Number of tracks       %n  Track number
   # %a  Artist                 %A  Album title
   # %y  Year                   %g  Genre
   # %t  Track title
   #****************************************************************
    def fileName(self):
        pattern = self._disc.pattern()
        if (not  pattern):
            pattern = "%n track"

        perc = False
        res = ''
        for c in (pattern):
            if (perc):
                perc = False
                if   (c == '%'): res += '%'; perc = True
                elif (c == 'N'): res += "%02d" % int(self._disc.trackCount())
                elif (c == 'n'): res += "%02d" % int(self.trackNum())
                elif (c == 'A'): res += self.safeStr(self.album())
                elif (c == 't'): res += self.safeStr(self.title())
                elif (c == 'a'): res += self.safeStr(self.artist())
                elif (c == 'y'):
                    if (self.year()) :
                        res += "%02d" % int(self.year())
                elif (c == 'g'): res += self.safeStr(self.genre())
                else:            res += '%' + c
            else:
                if (c == '%'):
                    perc = True
                else:
                    res += c

        return QString("%1.%2").arg(res).arg(self._disc.fileExtension())


   #****************************************************************
   #
   #****************************************************************
    def safeStr(self, str):
        res = str[:]
        res = res.replace('|', "-")
        res = res.replace('/', "-")
        res = res.replace('\\', "-")
        res = res.replace(':', "-")
        return res


   #****************************************************************
   #
   #****************************************************************
    def filePath(self):
        fileName = self.fileName()

        if (fileName == ''):
            return QString('')

        res = QFileInfo(fileName)

        if (res.isAbsolute()):
            return res.absoluteFilePath()

        # Add dir from trackList.outDir ===================
        outDir = QString(self._disc.outDir())
        if (not outDir.isEmpty()):
            if (outDir.startsWith("~/")):
                outDir.replace(0, 1, QDir.homePath())

            res = QFileInfo(outDir + "/" + res.filePath())

        if (res.isAbsolute()):
            return res.absoluteFilePath()

        # Add dir from trackList.audioFile file ===========
        af = QFileInfo(self._disc.audioFile()).dir().absolutePath()

        if (not af.isEmpty()):
            res = QFileInfo(af + "/" + res.filePath())

        return res.absoluteFilePath()


   #****************************************************************
   #
   #****************************************************************
    def progress(self):
        return self._progress


   #****************************************************************
   #
   #****************************************************************
    def status(self):
        return self._status


   #****************************************************************
   #
   #****************************************************************
    def setProgress(self, status, percent = -1):

        s2s = [# ------------ New statuses -----------
               # NoRun   Error  Abort   OK   Process |_____________
               [    0,     1,     1,     1,     1    ], # NoRun    |
               [    1,     0,     0,     0,     0    ], # Error    | Current
               [    1,     1,     0,     0,     0    ], # Aborted  | statuses
               [    1,     1,     0,     0,     0    ], # OK       |
               [    1,     1,     1,     1,     1    ], # Process  |
               ]

        # Process is some is Extracting, Encoding, Queued  etc.


        if not s2s[min(4, self._status)][min(4, status)]:
            return

        self._status = status
        self._progress = percent
        self._disc.trackChanged(self)


class Disc(QObject):
   #****************************************************************
   #
   #****************************************************************
    def __init__(self, parent = None):
        QObject.__init__(self, parent)
        self._tracks = []
        self._startTrackNum = 1

        self._cueFileName = QString()
        self._cueData = None

        self._codePage = 'UTF-8'

        self._currentRole = ''

        self._outDir = QString('')
        self._pattern = QString('')

        self._outFormat = AudioFormat.FLAC

        self._audioFile =None


   #****************************************************************
   #
   #****************************************************************
    def clearCueFile(self):
        self._cueFileName = QString()
        self._tracks = []
        self._startTrackNum = 1
        self._cueData = QStringList()
        self._fileTag = QString()


   #****************************************************************
   #
   #****************************************************************
    def trackChanged(self, track):
        index = track.index()
        self.emit(SIGNAL("dataChanged(int,int)"), index, index)


   #****************************************************************
   #
   #****************************************************************
    def tracks(self):
        return self._tracks


   #****************************************************************
   #
   #****************************************************************
    def trackCount(self):
        return len(self._tracks)


   #****************************************************************
   #
   #****************************************************************
    def track(self, num):
        return self._tracks[num]


   #****************************************************************
   #
   #****************************************************************
    def currentRole(self):
        return self._currentRole;


   #****************************************************************
   #
   #****************************************************************
    def setCurrentRole(self, role):
        self._currentRole = role


   #****************************************************************
   #
   #****************************************************************
    def cueFileName(self):
        return self._cueFileName


   #****************************************************************
   # Load and parse CUE file
   #****************************************************************
    def loadCueFile(self, fileName):
        self.clearCueFile()
        self._cueFileName = fileName

        if QString(fileName).isEmpty():
            self.emit(SIGNAL("reset()"))
            self.emit(SIGNAL("dataChanged(int,int)"), 0, -1)
            return

        try:
            FileError.checkFile(fileName)

            f = QFile(fileName)
            if (f.open(QIODevice.ReadOnly | QIODevice.Text)):
                self._cueData = QStringList()

                while not f.atEnd():
                    self._cueData.append(QString(f.readLine()))

                #****************************************************
                # Delete BOM
                # see http://en.wikipedia.org/wiki/Byte_Order_Mark
                #****************************************************
                if (self._cueData[0].startsWith("\xEF\xBB\xBF")):   # UTF-8
                    self._codePage = 'UTF-8'
                    self._cueData[0] = self._cueData[0].remove(0, 3)

                if (self._cueData[0].startsWith("\xFE\xFF")):       # UTF-16 (BE)
                    self._codePage = 'UTF-16BE'
                    self._cueData[0] = self._cueData[0].remove(0, 2)

                if (self._cueData[0].startsWith("\xFF\xFE")):       # UTF-16 (LE)
                    self._codePage = 'UTF-16LE'
                    self._cueData[0] = self._cueData[0].remove(0, 2)

                self._currentRole = self.tr('Cue file');
                self.parseCue(self._cueData)

                if not self.trackCount():
                    raise FileErrorBadCue(fileName)
        finally:
            self.emit(SIGNAL("reset()"))
            self.emit(SIGNAL("dataChanged(int,int)"), 0, -1)


   #****************************************************************
   #
   #****************************************************************
    def parseCue(self, lines):
        track  = QString()
        album  = QString()
        performer = QString()
        genre  = QString()
        discId = QString()
        year   = 0
        self._fileTag = QString()

        for i in xrange(0, lines.count()):
            line = lines[i].trimmed()

            if (line.isEmpty()): next
            line = line.trimmed()

            tag = line.section(' ', 0, 0).toUpper()
            value = line.section(' ', 1)
            if (value.length()>2 and (value[0] == "'" or value[0] == '"')):
                value = value.mid(1, value.length()-2)

            #==============================================
            if (tag == "TRACK"):
                num = value.section(' ', 0, 0)
                track = Track(self)
                self._tracks.append(track)
                track.setTrackNum(int(num))
                track.setArtist(performer, False)
                track.setAlbum(album, False)
                track.setGenre(genre, False)
                track.setYear(year, False)
                track.setDiscId(discId, False)

            #==============================================
            elif (tag == "TITLE"):
                if (track):
                    track.setTitle(value, False)
                else:
                    album = QString(value)

            #==============================================
            elif (tag == "PERFORMER"):
                if (track):
                    track.setArtist(value, False)
                else:
                    performer = QString(value)

            #==============================================
            elif (tag == "DATE"):           # For incorrect CUE files
                if (track):
                    track.setYear(value, False)
                else:
                    year = value

            #==============================================
            elif (tag == "FILE"):
                if not track:
                    self._fileTag = QString(value[:-5])

            #==============================================
            elif (tag == "REM"):
                subTag = value.section(' ', 0, 0).toUpper()
                value = value.section(' ', 1)
                if (value.length()>2 and (value[0] == "'" or value[0] == '"')):
                    value = value.mid(1, value.length()-2)

                if (not track):
                    if (subTag == 'DATE'):   year   = value
                    if (subTag == 'GENRE'):  genre  = value
                    if (subTag == 'DISCID'): discId = value



   #****************************************************************
   #
   #****************************************************************
    def codePage(self):
        return self._codePage


   #****************************************************************
   #
   #****************************************************************
    def setCodePage(self, value):
        self._codePage = str(value)
        self.emit(SIGNAL("dataChanged(int,int)"), 0, -1)


   #****************************************************************
   #
   #****************************************************************
    def startTrackNum(self):
        return self._startTrackNum


   #****************************************************************
   #
   #****************************************************************
    def setStartTrackNum(self, value):
        self._startTrackNum = max(1, int(value))
        self.emit(SIGNAL("dataChanged(int,int)"), 0, -1)


   #****************************************************************
   #
   #****************************************************************
    def pattern(self):
        return self._pattern


   #****************************************************************
   #
   #****************************************************************
    def setPattern(self, value):
        self._pattern = QString(value)
        self.emit(SIGNAL("dataChanged(int,int)"), 0, -1)


   #****************************************************************
   #
   #****************************************************************
    def outDir(self):
        return self._outDir


   #****************************************************************
   #
   #****************************************************************
    def setOutDir(self, value):
        self._outDir = QString(value)
        self.emit(SIGNAL("dataChanged(int,int)"), 0, -1)


   #****************************************************************
   #
   #****************************************************************
    def outFormat(self):
        return self._outFormat


   #****************************************************************
   #
   #****************************************************************
    def setOutFormat(self, value):
        self._outFormat = value
        self.emit(SIGNAL("dataChanged(int,int)"), 0, -1)


   #****************************************************************
   #
   #****************************************************************
    def fileExtension(self):
        if self._outFormat == AudioFormat.FLAC:  return QString('flac')
        if self._outFormat == AudioFormat.OGG:   return QString('ogg')
        if self._outFormat == AudioFormat.MP3:   return QString('mp3')
        if self._outFormat == AudioFormat.WAV:   return QString('wav')
        if self._outFormat == AudioFormat.WV:    return QString('wv')


   #****************************************************************
   #
   #****************************************************************
    def audioFileName(self):
        if self._audioFile:
            return self._audioFile.fileName()
        else:
            return QString()


   #****************************************************************
   #
   #****************************************************************
    def loadAudioFile(self, fileName):
        self._audioFile = None

        try:
            if not QString(fileName).isEmpty():
                self._audioFile = getAudioFile(fileName, self)
        finally:
            self.emit(SIGNAL("reset()"))
            self.emit(SIGNAL("dataChanged(int,int)"), 0, -1)


   #****************************************************************
   #
   #****************************************************************
    def findAudioFile(self, cueFileName):
        self._audioFile = None

        exts = QStringList()
        exts << "*.ape"
        exts << "*.wav"
        exts << "*.flac"
        exts << "*.wv"
        exts << "*.tta"

        fi = QFileInfo(cueFileName)

        patterns = QStringList()

        patterns << fi.completeBaseName()
        if not self._fileTag.isEmpty():
            patterns << QFileInfo(self._fileTag).completeBaseName()

        files = fi.dir().entryInfoList(exts, QDir.Files | QDir.Readable)
        for f in files:
           for pattern in patterns:
                if f.fileName().startsWith(pattern):
                    try:
                        self.loadAudioFile(f.absoluteFilePath())
                        return
                    except FileError, e:
                        pass


   #****************************************************************
   #
   #****************************************************************
    def findCueFile(self, audioFileName):
        self.clearCueFile()

        exts = QStringList()
        exts << "*.cue"

        fi = QFileInfo(audioFileName)
        pattern = fi.completeBaseName()

        files = fi.dir().entryInfoList(exts, QDir.Files | QDir.Readable)

        for f in files:
            if f.fileName().startsWith(pattern):
                try:
                    self.loadCueFile(f.absoluteFilePath())
                    return
                except FileError, e:
                    pass


   #****************************************************************
   #
   #****************************************************************
    def canConvert(self):
      #  print "CanConvert %s %s" % (self.trackCount() > 0 ,  self._audioFile)
        return self.trackCount() > 0 and  self._audioFile


   #****************************************************************
   #
   #****************************************************************
    def progressChanged(self, trackIndex, status, progress):
        if trackIndex > -1:
            self.track(trackIndex).setProgress(status, progress)


   #****************************************************************
   #
   #****************************************************************
    def cdQuality(self):
        return self._audioFile.cdQuality()


   #****************************************************************
   #
   #****************************************************************
    def cueData(self):
        return self._cueData

