# -*- coding: utf-8 -*-
# Project : flacon
# File : errors.py
# (c) Alexander Sokolov asokol@mail.ru
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.


import exceptions
#import subprocess
#import os

from PyQt4 import QtCore, QtGui
from PyQt4.QtCore import *

DEFAULT_CODEPAGE = 'UTF-8'


class Abort(Exception):
   #****************************************************************
   #
   #****************************************************************
    def __init__(self):
        Exception.__init__(self, '')


class FlaconException(Exception):
   #****************************************************************
   #
   #****************************************************************
    def __unicode__(self):
        return unicode(self.msg().toUtf8(), 'utf-8')


   #****************************************************************
   #
   #****************************************************************
    def tr(self, text, disambiguation = ''):
        return QtGui.qApp.translate(self.__class__.__name__, text, disambiguation)


   #****************************************************************
   #
   #****************************************************************
    def msg(self):
        return ""


#class CheckError(FlaconException):
   #****************************************************************
   #
   #****************************************************************
#    def __init__(self, msg):
#        FlaconException.__init__(self)
#        self._msg = msg

   #****************************************************************
   #
   #****************************************************************
#    def msg(self):
#        return self._msg


class FileError(FlaconException):
   #****************************************************************
   #
   #****************************************************************
    def __init__(self, fileName):
        FlaconException.__init__(self)
        self._fileName = fileName


   #****************************************************************
   #
   #****************************************************************
    def fileName(self):
        return self._fileName


   #****************************************************************
   #
   #****************************************************************
    def msg(self):
        return self.tr("The file <b>%1</b> could not be opened.").arg(self._fileName)


    #****************************************************************
    #
    #****************************************************************
    @staticmethod
    def checkFile(fileName, forWrite = False):
        fi = QFileInfo(fileName)

        if not fi.exists():     raise FileErrorNotFound(fileName)
        if not fi.isFile():     raise FileErrorNotRegular(fileName)
        if not fi.isReadable(): raise FileErrorAccessDenied(fileName)

        if forWrite:
            if not fi.isWritable(): raise FileErrorAccessDenied(fileName)



class FileErrorNotFound(FileError):
   #****************************************************************
   #
   #****************************************************************
    def msg(self):
        return self.tr("File <b>%1</b> not found.").arg(self._fileName)



class FileErrorNotRegular(FileError):
   #****************************************************************
   #
   #****************************************************************
    def msg(self):
        return self.tr("The file <b>%1</b> could not be opened. <br>File isn't regular file.").arg(self._fileName)



class FileErrorAccessDenied(FileError):
   #****************************************************************
   #
   #****************************************************************
    def msg(self):
        return self.tr("The file <b>%1</b> could not be opened. <br>Access denied.").arg(self._fileName)



class FileErrorBadAudio(FileError):
   #****************************************************************
   #
   #****************************************************************
    def msg(self):
        return self.tr("The file <b>%1</b> is not a supported audio file. <br><br>Verify that the all required programs are installed and in your preferences.").arg(self._fileName)



class FileErrorBadCue(FileError):
   #****************************************************************
   #
   #****************************************************************
    def msg(self):
        return self.tr("The file <b>%1</b> is not a valid CUE file.").arg(self._fileName)



