#!/usr/bin/python
# -*- coding: utf-8 -*-
# Project : flacon
# File : flacon.py
# (c) Alexander Sokolov asokol@mail.ru
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.


FLACON_VERSION='0.4.6'

import sys
import os
import string
import glob
import re

from PyQt4 import QtGui, QtCore
from PyQt4.QtCore import *
from mainWindow_ui import Ui_mainWindow

import aboutDialog
#from tracklist import *
from converter import *
from preferences import *
from configureDialog import *
from flaconglobal import *
from disc import *
from trackview import *
#from audiofiles import *

import pyqticonloader
import errors
import traceback


class Flacon(QMainWindow):
   #****************************************************************
   #
   #****************************************************************
    def __init__(self):
        QMainWindow.__init__(self)
        self.setWindowIcon(pyqticonloader.icon('flacon'))

        self.form = Ui_mainWindow()
        centralWidget = QtGui.QWidget(self)
        self.form.setupUi(centralWidget)
        self.setCentralWidget(centralWidget)
        self.setWindowTitle("Flacon")

        self._lastDir=''
        self._cueFileAuto = True
        self._audioFileAuto = True
        self._configureDialog = None


        self.disc = Disc()

        self.itemModel = DiscItemModel(self.form.trackListView)
        self.itemModel.setDisc(self.disc)
        self.itemModel.setIcon(pyqticonloader.icon('audio-x-generic'))
        self.itemModel.setView(self.form.trackListView)

        self.converter = Converter(self)

        self.form.trackListView.setModel(self.itemModel)
        self.form.trackListView.header().setStretchLastSection(True)
        self.form.trackListView.hideColumn(DiscItemModel.COLUMN_STATUS)
        self.form.trackListView.setContextMenuPolicy(Qt.ActionsContextMenu)

        self.initMenu()
        self.setAcceptDrops(True)

        self.connect(self.disc, SIGNAL('dataChanged(int, int)'), self.refreshEdits)
        self.connect(self.form.trackListView.selectionModel(), SIGNAL('currentChanged(QModelIndex, QModelIndex)'), self.refreshEdits)

        self.connect(self.form.cueFileEdit, SIGNAL('textEdited(QString)'), self.setCueFile)
        self.connect(self.form.audioFileEdit, SIGNAL('textEdited(QString)'), self.setAudioFile)

        self.connect(self.form.outPatternEdit, SIGNAL('textEdited(QString)'), self.disc.setPattern)
        self.connect(self.form.outDirEdit,     SIGNAL('textEdited(QString)'), self.disc.setOutDir)

        self.connect(self.form.tagGenreEdit,    SIGNAL('textEdited(QString)'),   self.setGenre)
        self.connect(self.form.tagYearEdit,     SIGNAL('valueChanged(QString)'), self.setYear)
        self.connect(self.form.tagArtistEdit,   SIGNAL('textEdited(QString)'),   self.setArtist)
        self.connect(self.form.tagAlbumEdit,    SIGNAL('textEdited(QString)'),   self.setAlbum)
        self.connect(self.form.tagStartNumEdit, SIGNAL('valueChanged(int)'), self.disc.setStartTrackNum)
        self.connect(self.form.tagDiscIdEdit,   SIGNAL('textEdited(QString)'),   self.setDiscId)

        self.connect(self.converter, SIGNAL('finished()'), self.setControlsEnable)
        self.connect(self.converter, SIGNAL('error(QString)'), self.error)
        self.connect(self._actAbort, SIGNAL('triggered()'), self.converter.stop)

        self.initCodepageCombo()
        self.initOutFormatCombo()
        self.initOutPatternButton()

        self.loadSettings()


   #****************************************************************
   #
   #****************************************************************
    def initOutFormatCombo(self):
        self.form.outFormatCombo.addItem(self.tr("FLAC"), QVariant(AudioFormat.FLAC))
        self.form.outFormatCombo.addItem(self.tr("WavPack"),  QVariant(AudioFormat.WV))
        self.form.outFormatCombo.addItem(self.tr("OGG"),  QVariant(AudioFormat.OGG))
        self.form.outFormatCombo.addItem(self.tr("WAV"),  QVariant(AudioFormat.WAV))
        self.form.outFormatCombo.addItem(self.tr("MP3"),  QVariant(AudioFormat.MP3))
        self.connect(self.form.outFormatCombo, SIGNAL('currentIndexChanged(int)'), self.setOutFormat)


   #****************************************************************
   #
   #****************************************************************
    def initCodepageCombo(self):
        self.form.codepageCombo.addItem(self.tr("Unicode (UTF-8)"),         QVariant("UTF-8"))
        self.form.codepageCombo.addItem(self.tr("Unicode (UTF-16LE)"),      QVariant("UTF-16LE"))
        self.form.codepageCombo.addItem(self.tr("Unicode (UTF-16BE)"),      QVariant("UTF-16BE"))
        self.form.codepageCombo.insertSeparator(99999)

        self.form.codepageCombo.addItem(self.tr("Cyrillic (Win-1251)"), QVariant("CP1251"))
        self.form.codepageCombo.addItem(self.tr("Cyrillic (CP-866)"),       QVariant("CP866"))
        self.connect(self.form.codepageCombo, SIGNAL('currentIndexChanged(int)'), self.setCodePage)


   #****************************************************************
   #
   #****************************************************************
    def initOutPatternButton(self):

        btn = self.form.outPatternButton
        btn.setPopupMode(QToolButton.InstantPopup)

        menu = QMenu(btn)

        act = QtGui.QAction(self.tr('Insert "Track number"'), self)
        self.connect(act, SIGNAL('triggered()'), (lambda : self.form.outPatternEdit.insert('%n')))
        menu.addAction(act)

        act = QtGui.QAction(self.tr('Insert "Total number of tracks"'), self)
        self.connect(act, SIGNAL('triggered()'), (lambda : self.form.outPatternEdit.insert('%N')))
        menu.addAction(act)

        act = QtGui.QAction(self.tr('Insert "Artist"'), self)
        self.connect(act, SIGNAL('triggered()'), (lambda : self.form.outPatternEdit.insert('%a')))
        menu.addAction(act)

        act = QtGui.QAction(self.tr('Insert "Album title"'), self)
        self.connect(act, SIGNAL('triggered()'), (lambda : self.form.outPatternEdit.insert('%A')))
        menu.addAction(act)

        act = QtGui.QAction(self.tr('Insert "Track title"'), self)
        self.connect(act, SIGNAL('triggered()'), (lambda : self.form.outPatternEdit.insert('%t')))
        menu.addAction(act)

        act = QtGui.QAction(self.tr('Insert "Year"'), self)
        self.connect(act, SIGNAL('triggered()'), (lambda : self.form.outPatternEdit.insert('%y')))
        menu.addAction(act)

        act = QtGui.QAction(self.tr('Insert "Genre"'), self)
        self.connect(act, SIGNAL('triggered()'), (lambda : self.form.outPatternEdit.insert('%g')))
        menu.addAction(act)

        btn.setMenu(menu)


   #****************************************************************
   #
   #****************************************************************
    def closeEvent(self, event):
        self.stopConvert()
        self.saveSettings()


   #****************************************************************
   #
   #****************************************************************
    def createMenuAction(self, iconName, caption, toolTip, shortcut, slot):
        result = QtGui.QAction(pyqticonloader.icon(iconName), caption, self)
        if shortcut:
            result.setShortcut(shortcut)
        if toolTip:
            result.setToolTip(toolTip)
        self.connect(result, SIGNAL('triggered()'), slot)
        return result


   #****************************************************************
   #
   #****************************************************************
    def initMenu(self):
        menubar = self.menuBar()

        mFile = menubar.addMenu(self.tr('&File'))

        getIcon = pyqticonloader.icon

        # Open CUE ........................................
        self._actOpenCueFile = QtGui.QAction(getIcon('document-open'), self.tr('Open CUE file'), self)
        self._actOpenCueFile.setShortcut('Ctrl+O')
        self.connect(self._actOpenCueFile, SIGNAL('triggered()'), self.openCueFile)
        mFile.addAction(self._actOpenCueFile)
        self.form.cueFileButton.setDefaultAction(self._actOpenCueFile)

        # Open audio ......................................
        self._actOpenAudioFile = QtGui.QAction(getIcon('document-open'), self.tr('Open audio file'), self)
        self._actOpenAudioFile.setShortcut('Ctrl+L')
        self.connect(self._actOpenAudioFile, SIGNAL('triggered()'), self.openAudioFile)
        mFile.addAction(self._actOpenAudioFile)
        self.form.audioFileButton.setDefaultAction(self._actOpenAudioFile)

        # Select out directory ............................
        self._actOpenOutDir = QtGui.QAction(getIcon(['document-open-folder', 'document-open']), self.tr('Select result directory'), self)
        self.connect(self._actOpenOutDir, SIGNAL('triggered()'), self.openOutDir)
        self.form.outDirButton.setDefaultAction(self._actOpenOutDir)

        #..................................................
        mFile.addSeparator()

        # Start convert ...................................
        self._actConvert = QtGui.QAction(getIcon('arrow-right'), self.tr('Convert'), self)
        self._actConvert.setShortcut('Ctrl+W')
        self._actConvert.setToolTip(self.tr('Start conversion process.'))
        self.connect(self._actConvert, SIGNAL('triggered()'), self.startConvert)
        self._actConvert.setEnabled(False)
        mFile.addAction(self._actConvert)
        self.form.runButton.setDefaultAction(self._actConvert)

        # Stop convert ....................................
        self._actAbort = QtGui.QAction(self.tr('Abort'), self)
        #self._actAbort.setShortcut('Ctrl+X')
        self._actAbort.setToolTip(self.tr('Abort conversion process.'))
        #self.connect(self._actAbort, SIGNAL('triggered()'), self.stopConvert)
        self._actAbort.setEnabled(False)
        mFile.addAction(self._actAbort)
        self.form.abortButton.setDefaultAction(self._actAbort)

        #..................................................
        mFile.addSeparator()

        # Exit ............................................
        actExit = QtGui.QAction(getIcon('application-exit'), self.tr('Exit'), self)
        actExit.setShortcut('Ctrl+Q')
        self.connect(actExit, SIGNAL('triggered()'), SLOT('close()'))
        mFile.addAction(actExit)


        mSettings = menubar.addMenu(self.tr('&Settings'))

        # Configure FLACON .................................
        self._actConfigure = QtGui.QAction(getIcon(['configure', 'preferences-system']), self.tr('&Configure flacon'), self)
        self._actConfigure.setShortcut('Ctrl+M')
        self.connect(self._actConfigure, SIGNAL('triggered()'), self.configure)
        mSettings.addAction(self._actConfigure)

        # Configure Encoder ................................
        self._actConfigureEncoder = QtGui.QAction(self._actConfigure.icon(), self.tr('Configure encoder'), self)
        self.connect(self._actConfigureEncoder, SIGNAL('triggered()'), self.configureEncoder)
        self.form.configureEncoderBtn.setDefaultAction(self._actConfigureEncoder)

        mHelp = menubar.addMenu(self.tr('&Help'))

        # About flacon ....................................
        actAbout = QtGui.QAction(getIcon('help-about'), self.tr('&About flacon'), self)
        self.connect(actAbout, SIGNAL('triggered()'), self.aboutFlacon)
        mHelp.addAction(actAbout)

        # About Qt ........................................
        #actAboutQt = QtGui.QAction(getIcon('help-about'), self.tr('&About Qt'), self)
        #self.connect(actAboutQt, SIGNAL('triggered()'), self.aboutQt)
        #mHelp.addAction(actAboutQt)


        # TrackView context menu ..........................
        self._actEditTrackTitle = QtGui.QAction(self.tr('Edit &Track title'), self)
        self.connect(self._actEditTrackTitle, SIGNAL('triggered()'), self.form.trackListView.editTitle)
        self._actEditTrackTitle.setShortcut('Ctrl+T')
        self.form.trackListView.addAction(self._actEditTrackTitle)

        self._actEditArtist = QtGui.QAction(self.tr('Edit &Artist name'), self)
        self.connect(self._actEditArtist, SIGNAL('triggered()'), self.form.trackListView.editArtist)
        self._actEditArtist.setShortcut('Ctrl+N')
        self.form.trackListView.addAction(self._actEditArtist)


   #****************************************************************
   # Load settings
   #****************************************************************
    def loadSettings(self):
        settings = QSettings("flacon", "flacon")

        # MainWindow geometry
        (width, ok)  = settings.value("MainWindow/Width",  QVariant(987)).toInt()
        (height, ok) = settings.value("MainWindow/Height", QVariant(450)).toInt()
        self.resize(width, height)

        # Splitter state
        self.form.splitter.restoreState(settings.value("MainWindow/Splitter").toByteArray())


        # Out file pattern & dir
        s = settings.value("OutFiles/Pattern", QVariant('%a/%y - %A/%n - %t')).toString()
        self.form.outPatternEdit.setText(s)
        self.disc.setPattern(s)

        s = settings.value("OutFiles/Directory", QVariant('~/music')).toString()
        self.form.outDirEdit.setText(s)
        self.disc.setOutDir(s)

        f = settings.value("OutFiles/Format", QVariant(AudioFormat.FLAC))
        n = self.form.outFormatCombo.findData(f)
        self.form.outFormatCombo.setCurrentIndex(n)

        self._lastDir = settings.value("Misc/LastDirectory", QVariant('~/')).toString()

        # CodePage
        s = settings.value("Tags/Codepage", QVariant('UTF-8'))
        n = self.form.codepageCombo.findData(QVariant(s))
        self.setCodePage(n)

        qApp.preferences.load(settings)


   #****************************************************************
   # Write settings
   #****************************************************************
    def saveSettings(self):
        settings = QSettings("flacon", "flacon")

        settings.setValue("MainWindow/Width", QVariant(self.size().width()))
        settings.setValue("MainWindow/Height", QVariant(self.size().height()))
        settings.setValue("MainWindow/Splitter", QVariant(self.form.splitter.saveState()))

        settings.setValue("OutFiles/Pattern", QVariant(self.form.outPatternEdit.text()))
        settings.setValue("OutFiles/Directory", QVariant(self.form.outDirEdit.text()))

        n = self.form.outFormatCombo.currentIndex()
        settings.setValue("OutFiles/Format", QVariant(self.form.outFormatCombo.itemData(n)))

        n = self.form.codepageCombo.currentIndex()
        settings.setValue("Tags/Codepage", QVariant(self.form.codepageCombo.itemData(n)))

        settings.setValue("Misc/LastDirectory", QVariant(self._lastDir))

        qApp.preferences.write(settings)


   #****************************************************************
   # Set artist tag for all selected tracks
   #****************************************************************
    def setArtist(self, value):
        for track in self.form.trackListView.selectedTracks():
            track.setArtist(value)


   #****************************************************************
   # Set album tag for all selected tracks
   #****************************************************************
    def setAlbum(self, value):
        for track in self.form.trackListView.selectedTracks():
            track.setAlbum(value)


   #****************************************************************
   # Set genre tag for all selected tracks
   #****************************************************************
    def setGenre(self, value):
        for track in self.form.trackListView.selectedTracks():
            track.setGenre(value)


   #****************************************************************
   # Set year tag for all selected tracks
   #****************************************************************
    def setYear(self, value):
        for track in self.form.trackListView.selectedTracks():
            (year, ok) = value.toInt()
            if not ok: year = 0

            track.setYear(year)


   #****************************************************************
   # Set discId tag for all selected tracks
   #****************************************************************
    def setDiscId(self, value):
        for track in self.form.trackListView.selectedTracks():
            track.setDiscId(value)


   #****************************************************************
   #
   #****************************************************************
    def setCodePage(self, index):
        if (index>-1):
            codec = self.form.codepageCombo.itemData(index).toString()
            self.disc.setCodePage(codec)


   #****************************************************************
   #
   #****************************************************************
    def setOutFormat(self, index):
        format = self.form.outFormatCombo.itemData(index).toString()
        self.disc.setOutFormat(format)
        self._actConfigureEncoder.setEnabled(format != AudioFormat.WAV)


   #****************************************************************
   #
   #****************************************************************
    def refreshEdits(self, index1 = 0, index2 = 0):
        index = self.form.trackListView.currentIndex()

        if (index.isValid()):
            track = self.disc.track(index.row())

            if (self.form.tagArtistEdit.text() != track.artist()):
                self.form.tagArtistEdit.setText(track.artist())

            if (self.form.tagAlbumEdit.text() !=  track.album()):
                self.form.tagAlbumEdit.setText(track.album())

            if (self.form.tagGenreEdit.text() != track.genre()):
                self.form.tagGenreEdit.setText(track.genre())

            if (self.form.tagYearEdit.value() != track.year()):
                self.form.tagYearEdit.setText(track.year())

            if (self.form.tagDiscIdEdit.text() != track.discId()):
                self.form.tagDiscIdEdit.setText(track.discId())
        else:
            self.form.tagArtistEdit.clear()
            self.form.tagAlbumEdit.clear()
            self.form.tagGenreEdit.clear()
            self.form.tagYearEdit.clear()
            self.form.tagDiscIdEdit.clear()


        if (self.form.tagStartNumEdit.value() != self.disc.startTrackNum()):
            self.form.tagStartNumEdit.setValue(self.disc.startTrackNum())

        if (self.form.outDirEdit.text() != self.disc.outDir()):
            self.form.outDirEdit.setText(self.disc.outDir())

        n = self.form.codepageCombo.findData(QVariant(self.disc.codePage()))
        if (self.form.codepageCombo.currentIndex() != n):
            self.form.codepageCombo.setCurrentIndex(n)

        self.setControlsEnable()


   #****************************************************************
   #
   #****************************************************************
    def openCueFile(self):
        flt = self.tr("CUE files")      + " (*.cue);;" +\
              self.tr("All files")      + " (*.*)"

        fileName = QFileDialog.getOpenFileName(self, self.tr("Open CUE file"), self._lastDir, flt)
        if (fileName):
            self._lastDir = QFileInfo(fileName).dir().path()
            self.setCueFile(fileName, True)


   #****************************************************************
   #
   #****************************************************************
    def openAudioFile(self):
        flt = self.tr("All supported formats")  + " (*.ape *.wav *.flac *.wv *.tta);;" +\
              self.tr("APE files")              + " (*.ape);;"  + \
              self.tr("FLAC files")             + " (*.flac);;" + \
              self.tr("WavPack files")          + " (*.wv);;"   + \
              self.tr("TTA files")              + " (*.tta);;"  + \
              self.tr("WAV files")              + " (*.wav);;"  + \
              self.tr("All files")              + " (*.*)"

        fileName = QFileDialog.getOpenFileName(self, self.tr("Open audio file"), self._lastDir, flt)
        if (fileName):
            self._lastDir = QFileInfo(fileName).dir().path()
            self.setAudioFile(fileName, True)


   #****************************************************************
   #
   #****************************************************************
    def openFile(self, fileName):
        if  QFileInfo(fileName).size() > 102400:
            self.setAudioFile(fileName, True)
        else:
            self.setCueFile(fileName, True)


   #****************************************************************
   #
   #****************************************************************
    def setCueFile(self, fileName, verbose=False):
        self._cueFileAuto = False
        error  = None
        QApplication.setOverrideCursor(Qt.WaitCursor)

        try:
            self.disc.loadCueFile(fileName)
            self.form.cueFileEdit.setText(fileName)

        except FileError, e:
            error = e

        except Exception, e:
            traceback.print_exc()
            error = e

        if (self._audioFileAuto or self.form.audioFileEdit.text().isEmpty()):
            self._audioFileAuto = True
            if error:
                self.disc.loadAudioFile('')
                self.form.audioFileEdit.setText('')
            else:
                self.disc.findAudioFile(fileName)
                self.form.audioFileEdit.setText(self.disc.audioFileName())

        QApplication.restoreOverrideCursor()

        if verbose and error:
            self.error(unicode(e))


   #****************************************************************
   #
   #****************************************************************
    def setAudioFile(self, fileName, verbose=False):
        self._audioFileAuto = False
        error  = None
        QApplication.setOverrideCursor(Qt.WaitCursor)

        try:
            self.disc.loadAudioFile(fileName)
            self.form.audioFileEdit.setText(fileName)

        except FileError, e:
            error = e

        except Exception, e:
            traceback.print_exc()
            error = e

        if (self._cueFileAuto or self.form.cueFileEdit.text().isEmpty()):
            self._cueFileAuto = True
            if error:
                self.disc.loadCueFile('')
                self.form.cueFileEdit.setText('')
            else:
                self.disc.findCueFile(fileName)
                self.form.cueFileEdit.setText(self.disc.cueFileName())

        QApplication.restoreOverrideCursor()

        if verbose and error:
            self.error(unicode(e))


   #****************************************************************
   #
   #****************************************************************
    def openOutDir(self):
        outDir = QFileDialog.getExistingDirectory(self, self.tr("Select result directory"), self.form.outDirEdit.text())
        if (outDir):
            self.disc.setOutDir(outDir)


   #****************************************************************
   #
   #****************************************************************
    def startConvert(self):
        for track in self.disc.tracks():
            track.setProgress(Statuses.NotRunning, -1)

        self.form.trackListView.showColumn(DiscItemModel.COLUMN_STATUS)
        self.form.trackListView.setColumnWidth(1, 150)

        self.converter.start(self.disc)
        self.setControlsEnable()


   #****************************************************************
   #
   #****************************************************************
    def stopConvert(self):
        self.converter.stop()
        self.setControlsEnable()


   #****************************************************************
   #
   #****************************************************************
    def setControlsEnable(self):
        running = self.converter.isRunning()
        notLoad = not self.disc.canConvert()

        self.form.inFilesBox.setDisabled(running)
        self.form.outFilesBox.setDisabled(running)
        self.form.tagsBox.setDisabled(running or notLoad)

        self._actOpenCueFile.setDisabled(running)
        self._actOpenAudioFile.setDisabled(running)

        self._actConfigure.setDisabled(running)
        self._actAbort.setEnabled(running)
        self._actConvert.setDisabled(running or notLoad)

        self._actEditTrackTitle.setDisabled(running or notLoad)
        self._actEditArtist.setDisabled(running or notLoad)


   #****************************************************************
   #
   #****************************************************************
    def error(self, msg):
        QMessageBox.critical(self,  self.tr("Error"), QString(msg))


   #****************************************************************
   #
   #****************************************************************
    def aboutFlacon(self):
        about = aboutDialog.AboutDialog()
        about.exec_()


   #****************************************************************
   #
   #****************************************************************
    def configureEncoder(self):
        format = self.form.outFormatCombo.itemData(self.form.outFormatCombo.currentIndex())
        index = -1
        if format == AudioFormat.FLAC: index = ConfigureDialog.flacPageIndex
        if format == AudioFormat.WV:   index = ConfigureDialog.wvPageIndex
        if format == AudioFormat.OGG:  index = ConfigureDialog.oggPageIndex
        if format == AudioFormat.MP3:  index = ConfigureDialog.mp3PageIndex
        self.configure(index)


   #****************************************************************
   #
   #****************************************************************
    def configure(self, pageIndex=-1):
        if not self._configureDialog:
            self._configureDialog = ConfigureDialog(self)

        self._configureDialog.show(pageIndex)
        self._configureDialog.raise_()
        self._configureDialog.activateWindow()
        self.connect(self._configureDialog, SIGNAL('finished(int)'), self.closeConfigureDialog)


   #****************************************************************
   #
   #****************************************************************
    def closeConfigureDialog(self):
        if not self._configureDialog:
            self._configureDialog = None


   #****************************************************************
   #
   #****************************************************************
    def dragEnterEvent(self, event):
        if event.mimeData().hasUrls():
             event.acceptProposedAction()


   #****************************************************************
   #
   #****************************************************************
    def dropEvent(self, event):
        if len(event.mimeData().urls()):
            self.openFile(event.mimeData().urls()[0].path())


#*******************************************************************
#
#*******************************************************************
if __name__ == "__main__":
    app = QtGui.QApplication(sys.argv)
    qApp.preferences = Preferences()

    translator = QTranslator(app)
    translator.load('flacon_' + QLocale.system().name(), modulesDir()+'/translations')
    app.installTranslator(translator)

    window = Flacon()
    window.show()
    args = app.arguments()
    if args.count() > 1:
        window.openFile(args[args.count()-1])

    sys.exit(app.exec_())
