# -*- coding: utf-8 -*-
# Project : flacon
# File : preferences.py
# (c) Alexander Sokolov asokol@mail.ru
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.


from PyQt4 import QtGui, QtCore
from PyQt4.QtCore import *
import subprocess
import flaconglobal

class Preferences():
   #****************************************************************
   #
   #****************************************************************
    def __init__(self):
        self._values = {}

        self.programs = ("shntool",
                         "flac",
                         "mac",
                         "oggenc",
                         "lame",
                         "wavpack",
                         "wvgain",
                         "metaflac",
                         "vorbisgain",
                         "mp3gain",
                         "ttaenc"
                        )

        # Globals *************************************************
        self._values['Encoder/ThreadCount'] = 8
        self._values['Encoder/TmpDir'] = ''

        # Programs *************************************************
        for program in self.programs:
            self._values['Programs/' + program ]=''

        # FLAC *****************************************************
        self._values['Flac/Compression'] = 5
        self._values['Flac/ReplayGain']  = flaconglobal.ReplayGainTypes.Disable

        # OGG ******************************************************
        self._values['Ogg/UseQuality']  = True
        self._values['Ogg/Quality']     = 7
        self._values['Ogg/MinBitrate']  = ''
        self._values['Ogg/NormBitrate'] = ''
        self._values['Ogg/MaxBitrate']  = ''
        self._values['Ogg/ReplayGain']  = flaconglobal.ReplayGainTypes.Disable

        # MP3 ******************************************************
        self._values['Mp3/Preset']  = 'avrStandard'
        self._values['Mp3/Bitrate'] = '320'
        self._values['Mp3/Quality'] = 4
        self._values['Mp3/ReplayGain']  = flaconglobal.ReplayGainTypes.Disable

        # WavPack **************************************************
        self._values['WavPack/Compression']  = 1
        self._values['WavPack/ReplayGain']  = flaconglobal.ReplayGainTypes.Disable


   #****************************************************************
   # Load settings.
   #****************************************************************
    def load(self, settings):
        for key, val in self._values.iteritems():
            self._values[key] = settings.value(key, QVariant(val))
        self.findPrograms()


   #****************************************************************
   # Write settings.
   #****************************************************************
    def write(self, settings):
        for key, val in self._values.iteritems():
            settings.setValue(key, QVariant(val))


   #****************************************************************
   # Returns the value for setting key.
   #****************************************************************
    def value(self, key):
        if self._values.has_key(key):
            return QVariant(self._values[key])
        else:
            return QVariant()


   #****************************************************************
   #
   #****************************************************************
    def valueAsBool(self, key):
        return self.value(key).toBool()


   #****************************************************************
   # Sets the value of setting key to value.
   #****************************************************************
    def setValue(self, key, value):
        self._values[key]=QVariant(value)


   #****************************************************************
   #
   #****************************************************************
    def findPrograms(self):
        for program in self.programs:
            if (self.value('Programs/'+program).toString().isEmpty()):
                path = subprocess.Popen(["which", program], stdout=subprocess.PIPE, stderr=subprocess.PIPE).communicate()[0]
                self.setValue('Programs/'+program, QString(path).trimmed())


   #****************************************************************
   #
   #****************************************************************
    def checkProgram(self, programKey):
        val = self.value(programKey).toString()
        if val.isEmpty():
            return False

        return QFileInfo(val).exists()
