# -*- coding: utf-8 -*-
# Project : flacon
# File : trackview.py
# (c) Alexander Sokolov asokol@mail.ru
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.


import os
import re

from PyQt4 import QtGui, QtCore
from PyQt4.QtCore import *
from PyQt4.QtGui import *

from preferences import *
from flaconglobal import *

class StateDelegate(QtGui.QItemDelegate):
   #****************************************************************
   #
   #****************************************************************
    def __init__(self, parent):
        QtGui.QItemDelegate.__init__(self, parent)


   #****************************************************************
   #
   #****************************************************************
    def paint(self, painter, option, index):
        #if (option.state and QtGui.QStyle.State_Selected):
        #    painter.fillRect(option.rect, option.palette.highlight());
        #QtGui.QItemDelegate.paint(self, painter, option, index)


        opt = QtGui.QStyleOptionProgressBar()
        opt.rect = option.rect.adjusted(4, 4, -4, -4)
        opt.minimum = 0
        opt.maximum = 100
        opt.progress = index.model().data(index, DiscItemModel.StatusPercentRole)

        status = index.model().data(index, DiscItemModel.StatusRole)
        if   status == Statuses.NotRunning: s = QString("")
        elif status == Statuses.Error:      s = self.tr("Error")
        elif status == Statuses.Aborted:    s = self.tr("Aborted")
        elif status == Statuses.OK:         s = self.tr("OK")
        elif status == Statuses.Extracting: s = self.tr("Extracting")
        elif status == Statuses.Encoding:   s = self.tr("Encoding")
        elif status == Statuses.Queued:     s = self.tr("Queued")
        elif status == Statuses.CalcGain:   s = self.tr("Calculate gain")
        elif status == Statuses.WaitGain:   s = self.tr("Wait gain")
        elif status == Statuses.WriteGain:  s = self.tr("Write gain")
        else: s = QString("Unknown")

        if (opt.progress < 0):
            opt.text = s
        else:
            opt.text = QString("%1 %2%").arg(s).arg(opt.progress)

        if (s != ''):
            QtGui.QApplication.style().drawControl(QtGui.QStyle.CE_ProgressBarContents, opt, painter, None)
            QtGui.QApplication.style().drawControl(QtGui.QStyle.CE_ProgressBarLabel, opt, painter, None)



class TracksView(QTreeView):
   #****************************************************************
   #
   #****************************************************************
    def __init__(self, parent):
        QTreeView.__init__(self, parent)
        delegate = StateDelegate(self)
        self.setItemDelegateForColumn(DiscItemModel.COLUMN_STATUS, delegate)


   #****************************************************************
   #
   #****************************************************************
    def selectedTracks(self):
        tracks = []
        for index in self.selectionModel().selectedIndexes():
            tracks.append(index.internalPointer())

        if (not len(tracks)): # No tracks was selected, return all tracks
            model = self.model()
            noIndex = QModelIndex()
            for row in xrange(0, model.rowCount(noIndex)):
                tracks.append(model.index(row, 0, noIndex).internalPointer())


        return tracks


   #****************************************************************
   # Open treeview inline editor for track title
   #****************************************************************
    def editTitle(self):
        index = QModelIndex(self.currentIndex())
        if (index.isValid()):
            index = index.sibling(index.row(), DiscItemModel.COLUMN_TITLE)
            self.edit(index)


   #****************************************************************
   # Open treeview inline editor for artist
   #****************************************************************
    def editArtist(self):
        index = QModelIndex(self.currentIndex())
        if (index.isValid()):
            index = index.sibling(index.row(), DiscItemModel.COLUMN_ARTIST)
            self.edit(index)


   #****************************************************************
   #
   #****************************************************************
    def reset(self):
        QTreeView.reset(self)

        index = self.currentIndex()
        if (not index.isValid()):
            index = self.model().index(0, 0, QModelIndex())
            self.setCurrentIndex(index)
            self.clearSelection()

        # Set columns width .......................................
        for i in xrange(0, self.header().count()):
            self.resizeColumnToContents(i)
            self.setColumnWidth(i, self.columnWidth(i)+12)



class DiscItemModel(QAbstractItemModel):
    StatusPercentRole = Qt.UserRole
    StatusRole = StatusPercentRole + 1

    COLUMN_TRACKNUM = 0
    COLUMN_STATUS   = 1
    COLUMN_TITLE    = 2
    COLUMN_ARTIST   = 3
    COLUMN_FILENAME = 4
    COLUMNS_COUNT   = 5

   #****************************************************************
   #
   #****************************************************************
    def __init__(self, parent=None):
        QAbstractItemModel.__init__(self, parent)
        self._disc = None
        self._icon = None
        self._view = None


   #****************************************************************
   #
   #****************************************************************
    def view(self):
        return self._view


   #****************************************************************
   #
   #****************************************************************
    def setView(self, value):
        self._view = value


   #****************************************************************
   #
   #****************************************************************
    def disc(self):
        return self._disc


   #****************************************************************
   #
   #****************************************************************
    def setDisc(self, disc):
        self._disc = disc
        self.connect(self._disc, SIGNAL("dataChanged(int,int)"), self.refresh)
        self.connect(self._disc, SIGNAL("reset()"), self.reset)


   #****************************************************************
   #
   #****************************************************************
    def icon(self):
        return self._icon


   #****************************************************************
   #
   #****************************************************************
    def setIcon(self, icon):
        self._icon = icon
        self.refresh()

   #****************************************************************
   #
   #****************************************************************
    def columnCount(self, parent):
        return self.COLUMNS_COUNT


   #****************************************************************
   #
   #****************************************************************
    def rowCount(self, parent):
        if self._disc:
            return self._disc.trackCount()
        else:
            return 0


   #****************************************************************
   #
   #****************************************************************
    def headerData(self, section, orientation, role):
        if orientation == Qt.Horizontal and role == Qt.DisplayRole:
            if (section == self.COLUMN_TRACKNUM): return QVariant(self.tr("Track"))
            if (section == self.COLUMN_STATUS):   return QVariant(self.tr("Status"))
            if (section == self.COLUMN_TITLE):    return QVariant(self.tr("Title"))
            if (section == self.COLUMN_ARTIST):   return QVariant(self.tr("Artist"))
            if (section == self.COLUMN_FILENAME): return QVariant(self.tr("File"))

        return QVariant()


   #****************************************************************
   #
   #****************************************************************
    def index(self, row, column, parent):
        if row < 0 or column < 0 or row >= self.rowCount(parent) or column >= self.columnCount(parent):
            return QModelIndex()

        return self.createIndex(row, column, self._disc.tracks()[row])


   #****************************************************************
   #
   #****************************************************************
    def parent(self, index):
        return QModelIndex()


   #****************************************************************
   #
   #****************************************************************
    def flags(self, index):
        if not index.isValid():
            return Qt.ItemIsEnabled

        res = Qt.ItemIsEnabled | Qt.ItemIsSelectable
        if (index.column() == self.COLUMN_TITLE or
            index.column() == self.COLUMN_ARTIST
           ):
            res = res | Qt.ItemIsEditable

        return res


   #****************************************************************
   #
   #****************************************************************
    def data(self, index, role):
        if not index.isValid():
            return QVariant()

        track = index.internalPointer()

        # =================================================
        if (role == Qt.DisplayRole or role == Qt.EditRole ):

            if (index.column() == self.COLUMN_TRACKNUM): return QVariant("%02d" % track.trackNum())
            if (index.column() == self.COLUMN_STATUS):   return QVariant('')
            if (index.column() == self.COLUMN_TITLE):    return QVariant(track.title())
            if (index.column() == self.COLUMN_ARTIST):   return QVariant(track.artist())
            if (index.column() == self.COLUMN_FILENAME): return QVariant(track.fileName())

        # =================================================
        if (role == Qt.ToolTipRole):
            if (index.column() == self.COLUMN_FILENAME): return QVariant(track.filePath())

        # =================================================
        elif (role == Qt.DecorationRole):
            if (index.column() == self.COLUMN_TRACKNUM): return QVariant(self._icon)

        # =================================================
        elif (role == self.StatusPercentRole):
            return track.progress()

        # =================================================
        elif (role == self.StatusRole):
            return track.status()


        return QVariant()

   #****************************************************************
   #
   #****************************************************************
    def setData(self, index, value, role):
        if not index.isValid():
            return False

        if role != Qt.EditRole:
            return False

        indexes = self._view.selectionModel().selectedIndexes()
        for ind in indexes:
            track = ind.internalPointer()

            if (index.column() == self.COLUMN_TITLE):
                track.setTitle(value.toString())

            elif (index.column() == self.COLUMN_ARTIST):
                track.setArtist(value.toString())

            #else:
                #return False

        self.refresh()
        return True


   #****************************************************************
   #
   #****************************************************************
    def refresh(self, row1=0, row2=-1):
        if row2<0:
            row2 = self.rowCount(QModelIndex())

        cols = self.columnCount(QModelIndex())
        index1 = self.index(row1, 0, QModelIndex())
        index2 = self.index(row2, cols-1, QModelIndex)
        self.emit(SIGNAL("dataChanged(QModelIndex, QModelIndex)"), index1, index2)
